@@ -433,6 +433,10 @@ static PyObject *qtmsghandler = nullptr;
 static void msgHandlerCallback(QtMsgType type, const QMessageLogContext &ctx, const QString &msg)
 {
     Shiboken::GilState state;
+    PyObject *excType{};
+    PyObject *excValue{};
+    PyObject *excTraceback{};
+    PyErr_Fetch(&excType, &excValue, &excTraceback);
     Shiboken::AutoDecRef arglist(PyTuple_New(3));
     PyTuple_SetItem(arglist, 0, %CONVERTTOPYTHON[QtMsgType](type));
     PyTuple_SetItem(arglist, 1, %CONVERTTOPYTHON[QMessageLogContext &](ctx));
@@ -440,6 +444,7 @@ static void msgHandlerCallback(QtMsgType type, const QMessageLogContext &ctx, co
     const char *data = array.constData();
     PyTuple_SetItem(arglist, 2, %CONVERTTOPYTHON[const char *](data));
     Shiboken::AutoDecRef ret(PyObject_CallObject(qtmsghandler, arglist));
+    PyErr_Restore(excType, excValue, excTraceback);
 }
 // @snippet qt-messagehandler
 
@@ -1292,6 +1297,18 @@ Py_BEGIN_ALLOW_THREADS
 Py_END_ALLOW_THREADS
 // @snippet qdebug-format-string
 
+// @snippet qmessagelogger-format-string
+Py_BEGIN_ALLOW_THREADS
+%CPPSELF->%FUNCTION_NAME("%s", %1); // Uses placeholder for security reasons
+Py_END_ALLOW_THREADS
+// @snippet qmessagelogger-format-string
+
+// @snippet qmessagelogger-logcategory-format-string
+Py_BEGIN_ALLOW_THREADS
+%CPPSELF->%FUNCTION_NAME(%1, "%s", %2); // Uses placeholder for security reasons
+Py_END_ALLOW_THREADS
+// @snippet qmessagelogger-logcategory-format-string
+
 // @snippet qresource-registerResource
  auto ptr = reinterpret_cast<uchar *>(Shiboken::Buffer::getPointer(%PYARG_1));
  %RETURN_TYPE %0 = %CPPSELF.%FUNCTION_NAME(const_cast<const uchar *>(ptr), %2);
@@ -1746,18 +1763,51 @@ pthread_cleanup_pop(0);
 #endif
 // @snippet qthread_pthread_cleanup_uninstall
 
-// @snippet qlibraryinfo_build
-auto oldResult = pyResult;
-const auto version = _PepRuntimeVersion();
-pyResult = PyUnicode_FromFormat(
-#ifdef Py_LIMITED_API
-                                "%U [Python limited API %d.%d.%d]",
+// @snippet qlibraryinfo_python_build
+
+// For versions with one byte per digit.
+static QByteArray versionString(long version)
+{
+    return QByteArray::number((version >> 16) & 0xFF)
+           + '.' + QByteArray::number((version >> 8) & 0xFF)
+           + '.' + QByteArray::number(version & 0xFF);
+}
+
+static QByteArray pythonBuild()
+{
+    using namespace Qt::StringLiterals;
+
+#ifdef PYPY_VERSION
+    QByteArray result = "PyPy " PYPY_VERSION
 #else
-                                "%U [Python %d.%d.%d]",
+    QByteArray result = "Python"
 #endif
-                                oldResult, (version >> 16) & 0xFF,
-                                (version >> 8) & 0xFF, version & 0xFF);
-Py_DECREF(oldResult);
+#ifdef Py_LIMITED_API
+    " limited API"
+#endif
+#ifdef Py_GIL_DISABLED
+    " free threaded"
+#endif
+        ;
+    result += ' ';
+
+    const auto runTimeVersion = _PepRuntimeVersion();
+    const auto runTimeVersionB = versionString(runTimeVersion);
+    constexpr long buildVersion = PY_VERSION_HEX >> 8;
+    if (runTimeVersion == buildVersion) {
+        result += runTimeVersionB;
+    } else {
+        result += "run time: "_ba + runTimeVersionB + " built: "_ba
+                  + versionString(buildVersion);
+    }
+    return result;
+}
+// @snippet qlibraryinfo_python_build
+
+// @snippet qlibraryinfo_build
+QByteArray %0 = %CPPSELF.%FUNCTION_NAME();
+%0 += " [" + pythonBuild() + ']';
+%PYARG_0 = PyUnicode_FromString(%0.constData());
 // @snippet qlibraryinfo_build
 
 // @snippet qsharedmemory_data_readonly
